@@ -1,5 +1,5 @@
 from datetime import datetime
-from typing import Any, List, Tuple
+from typing import Any, Callable, List, Tuple
 
 from uplink import (
     Body,
@@ -25,6 +25,7 @@ from bpkio_api.helpers.list import (
 from bpkio_api.helpers.objects import find_duplicates_of
 from bpkio_api.helpers.search import SearchMethod, search_array_with_filters
 from bpkio_api.helpers.upsert import UpsertOperationType, upsert_status
+from bpkio_api.mappings import to_input_model
 from bpkio_api.models import (
     ContentReplacementSlot,
     VirtualChannelSlot,
@@ -87,7 +88,11 @@ class ServicesApi(BpkioSdkConsumer):
     # === Helpers ===
 
     @cache_api_results("list_services", key_suffix_from_arg="sparse")
-    def list(self, sparse: bool = True):
+    def list(
+        self,
+        sparse: bool = True,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ):
         """List all services
 
         Args:
@@ -96,12 +101,16 @@ class ServicesApi(BpkioSdkConsumer):
         Returns:
             List[Svc.ServiceSparse | Svc.AdInsertionService | Svc.ContentReplacementService | Svc.VirtualChannelService | Svc.AdaptiveStreamingCdnService]: List of services
         """
-        sparse_services = get_all_with_pagination(self._get_page, count_fn=self.count)
+        sparse_services = get_all_with_pagination(
+            self._get_page, count_fn=self.count, progress_hook=progress_hook
+        )
         if sparse:
             return sparse_services
         else:
             return collect_from_sparse_resources(
-                sparse_resources=sparse_services, get_fn=self._get_by_type
+                sparse_resources=sparse_services,
+                get_fn=self._get_by_type,
+                progress_hook=progress_hook,
             )
 
     def search_by_type(self, type: Svc.ServiceType) -> List[Svc.ServiceSparse]:
@@ -114,6 +123,8 @@ class ServicesApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        sparse: bool = True,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[Svc.ServiceSparse]:
         """Searches the list of services for those matching a particular filter query
 
@@ -139,7 +150,7 @@ class ServicesApi(BpkioSdkConsumer):
         if not filters:
             filters = [(value, field, method)]
 
-        services = self.list()
+        services = self.list(sparse=sparse, progress_hook=progress_hook)
         return search_array_with_filters(services, filters=filters)
 
     def _get_by_type(
@@ -327,7 +338,8 @@ class ServicesApi(BpkioSdkConsumer):
         service = self.retrieve(service_id)
         service.state = "paused"
         endpoint = self._mappings(service)
-        return endpoint.update(service_id, service)
+        service_in = to_input_model(service)
+        return endpoint.update(service_id, service_in)
 
     def unpause(
         self, service_id: int
@@ -340,7 +352,8 @@ class ServicesApi(BpkioSdkConsumer):
         service = self.retrieve(service_id)
         service.state = "enabled"
         endpoint = self._mappings(service)
-        return endpoint.update(service_id, service)
+        service_in = to_input_model(service)
+        return endpoint.update(service_id, service_in)
 
 
 # === CONTENT-REPLACEMENT SERVICES ===
@@ -397,14 +410,26 @@ class ContentReplacementServiceApi(BpkioSdkConsumer):
         return self.parent_api.unpause(service_id)
 
     @cache_api_results("list_cr")
-    def list(self) -> List[Svc.ContentReplacementService]:
+    def list(
+        self,
+        sparse: bool = True,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ) -> List[Svc.ContentReplacementService]:
         """List all Content Replacement services"""
         sparse_vcs = self.parent_api.search(
             Svc.ServiceType.CONTENT_REPLACEMENT,
             field="type",
             method=SearchMethod.STRICT,
+            progress_hook=progress_hook,
+            sparse=True,
+        )
+        if sparse:
+            return sparse_vcs
+        return collect_from_ids(
+            ids=[vc.id for vc in sparse_vcs],
+            get_fn=self.retrieve,
+            progress_hook=progress_hook,
         )
-        return collect_from_ids(ids=[vc.id for vc in sparse_vcs], get_fn=self.retrieve)
 
     def search(
         self,
@@ -412,12 +437,13 @@ class ContentReplacementServiceApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[Svc.ContentReplacementService]:
         """Searches the list of Content Replacement services for those matching a particular filter query"""
         if not filters:
             filters = [(value, field, method)]
 
-        services = self.list()
+        services = self.list(progress_hook=progress_hook, sparse=False)
         return search_array_with_filters(services, filters=filters)
 
     def clear(self, service_id):
@@ -551,12 +577,26 @@ class VirtualChannelServiceApi(BpkioSdkConsumer):
         return self.parent_api.unpause(service_id)
 
     @cache_api_results("list_vcs")
-    def list(self) -> List[Svc.VirtualChannelService]:
+    def list(
+        self,
+        sparse: bool = True,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ) -> List[Svc.VirtualChannelService]:
         """List all Virtual Channel services"""
         sparse_vcs = self.parent_api.search(
-            Svc.ServiceType.VIRTUAL_CHANNEL, field="type", method=SearchMethod.STRICT
+            Svc.ServiceType.VIRTUAL_CHANNEL,
+            field="type",
+            method=SearchMethod.STRICT,
+            progress_hook=progress_hook,
+            sparse=True,
+        )
+        if sparse:
+            return sparse_vcs
+        return collect_from_ids(
+            ids=[vc.id for vc in sparse_vcs],
+            get_fn=self.retrieve,
+            progress_hook=progress_hook,
         )
-        return collect_from_ids(ids=[vc.id for vc in sparse_vcs], get_fn=self.retrieve)
 
     def search(
         self,
@@ -564,12 +604,13 @@ class VirtualChannelServiceApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[Svc.VirtualChannelService]:
         """Searches the list of Virtual Channel services for those matching a particular filter query"""
         if not filters:
             filters = [(value, field, method)]
 
-        services = self.list()
+        services = self.list(progress_hook=progress_hook, sparse=False)
         return search_array_with_filters(services, filters=filters)
 
     def clear(self, service_id):
@@ -686,7 +727,9 @@ class AdInsertionServiceApi(BpkioSdkConsumer):
     @returns.json(Svc.AdInsertionService)
     @put("services/ad-insertion/{service_id}")
     def update(
-        self, service_id: int, service: Body(type=Svc.AdInsertionServiceIn)
+        self,
+        service_id: int,
+        service: Body(type=Svc.AdInsertionServiceIn),
     ) -> Svc.AdInsertionService:  # type: ignore
         """Update an Ad Insertion service"""
 
@@ -711,12 +754,26 @@ class AdInsertionServiceApi(BpkioSdkConsumer):
         return self.parent_api.unpause(service_id)
 
     @cache_api_results("list_dai")
-    def list(self) -> List[Svc.AdInsertionService]:
+    def list(
+        self,
+        sparse: bool = True,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ) -> List[Svc.AdInsertionService]:
         """List all Ad Insertion services"""
         sparse_vcs = self.parent_api.search(
-            Svc.ServiceType.AD_INSERTION, field="type", method=SearchMethod.STRICT
+            Svc.ServiceType.AD_INSERTION,
+            field="type",
+            method=SearchMethod.STRICT,
+            progress_hook=progress_hook,
+            sparse=True,
+        )
+        if sparse:
+            return sparse_vcs
+        return collect_from_ids(
+            ids=[vc.id for vc in sparse_vcs],
+            get_fn=self.retrieve,
+            progress_hook=progress_hook,
         )
-        return collect_from_ids(ids=[vc.id for vc in sparse_vcs], get_fn=self.retrieve)
 
     def search(
         self,
@@ -724,12 +781,13 @@ class AdInsertionServiceApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[Svc.AdInsertionService]:
         """Searches the list of Ad Insertion services for those matching a particular filter query"""
         if not filters:
             filters = [(value, field, method)]
 
-        services = self.list()
+        services = self.list(progress_hook=progress_hook, sparse=False)
         return search_array_with_filters(services, filters=filters)
 
 
@@ -785,14 +843,26 @@ class AdaptiveStreamingCdnServiceApi(BpkioSdkConsumer):
         return self.parent_api.unpause(service_id)
 
     @cache_api_results("list_ascdn")
-    def list(self) -> List[Svc.AdaptiveStreamingCdnService]:
+    def list(
+        self,
+        sparse: bool = True,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
+    ) -> List[Svc.AdaptiveStreamingCdnService]:
         """List all Adaptive Streaming CDN services"""
         sparse_vcs = self.parent_api.search(
             Svc.ServiceType.ADAPTIVE_STREAMING_CDN,
             field="type",
             method=SearchMethod.STRICT,
+            progress_hook=progress_hook,
+            sparse=True,
+        )
+        if sparse:
+            return sparse_vcs
+        return collect_from_ids(
+            ids=[vc.id for vc in sparse_vcs],
+            get_fn=self.retrieve,
+            progress_hook=progress_hook,
         )
-        return collect_from_ids(ids=[vc.id for vc in sparse_vcs], get_fn=self.retrieve)
 
     def search(
         self,
@@ -800,10 +870,11 @@ class AdaptiveStreamingCdnServiceApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[Svc.AdaptiveStreamingCdnService]:
         """Searches the list of Adaptive Streaming CDN services for those matching a particular filter query"""
         if not filters:
             filters = [(value, field, method)]
 
-        services = self.list()
+        services = self.list(progress_hook=progress_hook, sparse=False)
         return search_array_with_filters(services, filters=filters)
