@@ -1,4 +1,4 @@
-from typing import Any, List, Tuple
+from typing import Any, Callable, List, Tuple
 
 from bpkio_api.caching import cache_api_results
 from bpkio_api.consumer import BpkioSdkConsumer
@@ -32,9 +32,9 @@ class UsersApi(BpkioSdkConsumer):
     # === Helpers ===
 
     @cache_api_results("list_users")
-    def list(self):
+    def list(self, progress_hook: Callable[[int, Any, str | None], None] | None = None, **kwargs):
         """List all users"""
-        return get_all_with_pagination(self._get_page, count_fn=self.count)
+        return get_all_with_pagination(self._get_page, count_fn=self.count, progress_hook=progress_hook)
 
     def search(
         self,
@@ -42,6 +42,7 @@ class UsersApi(BpkioSdkConsumer):
         field: str | None = None,
         method: SearchMethod = SearchMethod.STRING_SUB,
         filters: List[Tuple[Any, str | None, SearchMethod | None]] | None = None,
+        progress_hook: Callable[[int, Any, str | None], None] | None = None,
     ) -> List[User]:
         """Searches the list of users for those matching a particular filter query
 
