@@ -1,7 +1,8 @@
 from typing import List
 
+from pydantic import BaseModel, Field
+
 from bpkio_api.models.common import BaseResource, NamedModel
-from pydantic import BaseModel
 
 
 class SubCategory(BaseModel):
@@ -10,7 +11,7 @@ class SubCategory(BaseModel):
 
 
 class CategoryIn(NamedModel):
-    subcategories: List[SubCategory] = []
+    subcategories: List[SubCategory] = Field(default_factory=list)
 
     @property
     def subcategories_list(self):
